@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.45 2004/01/23 18:51:30 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.46.2.1 2004/05/04 13:52:30 fuyu Exp $
  */
 
 #include "config.h"
@@ -61,8 +61,7 @@ static void bbs_thread_view_init(BBSThreadView *view);
 static void bbs_thread_view_finalize(GObject *object);
 static void bbs_thread_view_destroy(GtkObject *object);
 
-static void text_view_realize_cb(GtkTextView *text_view,
-				 BBSThreadView *bbs_thread_view);
+static void text_view_realize_cb(GtkTextView *text_view, BBSThreadView *view);
 
 static gboolean text_view_get_iter_at_link_tag_from_event(
 						GtkTextView *text_view,
@@ -70,6 +69,8 @@ static gboolean text_view_get_iter_at_link_tag_from_event(
 						GdkEventAny *event);
 static gboolean bbs_thread_view_motion_notify(GtkWidget *widget,
 					      GdkEventMotion *event);
+static gboolean bbs_thread_view_leave_notify(GtkWidget *widget,
+					     GdkEventCrossing *event);
 static gboolean bbs_thread_view_button_press(GtkWidget *widget,
 					     GdkEventButton *event);
 static gboolean bbs_thread_view_button_release(GtkWidget *widget,
@@ -312,11 +313,10 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(BBSThreadViewClass, link_mouse_out),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__BOXED_POINTER_STRING,
-		 G_TYPE_NONE, 3,
+		 ochusha_marshal_VOID__BOXED_POINTER,
+		 G_TYPE_NONE, 2,
 		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE,
-		 G_TYPE_POINTER,
-		 G_TYPE_STRING);
+		 G_TYPE_POINTER);
   bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL] =
     g_signal_new("link_mouse_press",
 		 G_TYPE_FROM_CLASS(klass),
@@ -436,6 +436,7 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 			       "close_thread", 0);
 
   widget_class->motion_notify_event = bbs_thread_view_motion_notify;
+  widget_class->leave_notify_event = bbs_thread_view_leave_notify;
   widget_class->button_press_event = bbs_thread_view_button_press;
   widget_class->button_release_event = bbs_thread_view_button_release;
 
@@ -574,7 +575,7 @@ bbs_thread_view_destroy(GtkObject *object)
 
 
 static void
-text_view_realize_cb(GtkTextView *text_view, BBSThreadView *bbs_thread_view)
+text_view_realize_cb(GtkTextView *text_view, BBSThreadView *view)
 {
   GdkDisplay *display = gtk_widget_get_display(GTK_WIDGET(text_view));
   GdkWindow *window = gtk_text_view_get_window(text_view,
@@ -584,22 +585,19 @@ text_view_realize_cb(GtkTextView *text_view, BBSThreadView *bbs_thread_view)
 			 & ~GDK_POINTER_MOTION_HINT_MASK)
 			| GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 
-  if (bbs_thread_view->ibeam_cursor != NULL)
-    gdk_cursor_unref(bbs_thread_view->ibeam_cursor);
-  bbs_thread_view->ibeam_cursor = gdk_cursor_new_for_display(display,
-							     GDK_XTERM);
+  if (view->ibeam_cursor != NULL)
+    gdk_cursor_unref(view->ibeam_cursor);
+  view->ibeam_cursor = gdk_cursor_new_for_display(display, GDK_XTERM);
 
-  if (bbs_thread_view->hand_cursor != NULL)
-    gdk_cursor_unref(bbs_thread_view->hand_cursor);
-  bbs_thread_view->hand_cursor = gdk_cursor_new_for_display(display,
-							    GDK_HAND2);
+  if (view->hand_cursor != NULL)
+    gdk_cursor_unref(view->hand_cursor);
+  view->hand_cursor = gdk_cursor_new_for_display(display, GDK_HAND2);
 }
 
 
 static gboolean
-text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
-					  GtkTextIter *iter,
-					  GdkEventAny *event)
+text_view_get_iter_from_event(GtkTextView *text_view, GtkTextIter *iter,
+			      GdkEventAny *event)
 {
   int buf_x;
   int buf_y;
@@ -630,6 +628,17 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
     }
 
   gtk_text_view_get_iter_at_location(text_view, iter, buf_x, buf_y);
+  return TRUE;
+}
+
+
+static gboolean
+text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
+					  GtkTextIter *iter,
+					  GdkEventAny *event)
+{
+  if (!text_view_get_iter_from_event(text_view, iter, event))
+    return FALSE;
 
   if (gtk_text_iter_has_tag(iter, link_tags[0]))
     {	/* link_tags[0]ǰϤޤƤϰϤؤƤ */
@@ -684,39 +693,86 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
 }
 
 
+static gchar *
+bbs_thread_view_get_selected_text_under_pointer(BBSThreadView *view,
+						GtkTextIter *start,
+						GdkEventAny *event)
+{
+  GtkTextIter end;
+  GtkTextIter iter;
+
+  if (!gtk_text_buffer_get_selection_bounds(view->text_buffer,
+					    start, &end))
+    return NULL;	/* 򤵤Ƥʤ */
+
+  if (!text_view_get_iter_from_event(&view->text_view, &iter, event))
+    return NULL;
+
+  if (!gtk_text_iter_in_range(&iter, start, &end))
+    return NULL;
+
+  return gtk_text_iter_get_visible_text(start, &end);
+}
+
+
 static gboolean
 bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 {
   gboolean result = FALSE;
   GtkTextIter iter;
   GtkTextView *text_view;
-  BBSThreadView *bbs_thread_view;
+  BBSThreadView *view;
+  gchar *text;
 
   g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && event->type == GDK_MOTION_NOTIFY, FALSE);
 
   text_view = GTK_TEXT_VIEW(widget);
-  bbs_thread_view = BBS_THREAD_VIEW(widget);
+  view = BBS_THREAD_VIEW(widget);
 
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
+  text = bbs_thread_view_get_selected_text_under_pointer(view,
+							 &iter,
+							 (GdkEventAny *)event);
+  if (text != NULL)
     {
       int start_offset = gtk_text_iter_get_offset(&iter);
-      if (start_offset == bbs_thread_view->last_offset)
+      if (start_offset != view->last_offset)
 	{
-	  goto finished;
+	  if (view->last_offset != -1)
+	    emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	  view->last_offset = start_offset;
+
+	  gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view),
+							 GTK_TEXT_WINDOW_TEXT),
+				view->hand_cursor);
+
+	  g_signal_emit(G_OBJECT(view),
+			bbs_thread_view_signals[LINK_MOUSE_OVER_SIGNAL],
+			0,
+			event,
+			view->thread,
+			text);
 	}
 
-      if (bbs_thread_view->last_offset != -1)
-	emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
-
-      bbs_thread_view->last_offset = start_offset;
-      emit_link_mouse_over(bbs_thread_view, event);
+      G_FREE(text);
     }
-  else if (bbs_thread_view->last_offset != -1)
-    emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
+  else if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
+						     (GdkEventAny *)event))
+    {
+      int start_offset = gtk_text_iter_get_offset(&iter);
+      if (start_offset != view->last_offset)
+	{
+	  if (view->last_offset != -1)
+	    emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	  view->last_offset = start_offset;
+	  emit_link_mouse_over(view, event);
+	}
+    }
+  else if (view->last_offset != -1)
+    emit_link_mouse_out(view, (GdkEventAny *)event);
 
- finished:
   if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
     result = (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
 								    event);
@@ -725,6 +781,21 @@ bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 }
 
 
+static gboolean
+bbs_thread_view_leave_notify(GtkWidget *widget, GdkEventCrossing *event)
+{
+  BBSThreadView *view = BBS_THREAD_VIEW(widget);
+
+  if (view->last_offset != -1)
+    emit_link_mouse_out(view, (GdkEventAny *)event);
+
+  if (GTK_WIDGET_CLASS(parent_class)->leave_notify_event != NULL)
+    return (*GTK_WIDGET_CLASS(parent_class)->leave_notify_event)(widget,
+								 event);
+  return FALSE;
+}
+
+
 static void
 emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
 {
@@ -757,21 +828,11 @@ emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
 static void
 emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 {
-  gchar *link;
   if (view->last_offset == -1)
     return;
 
-  if (view->link_table == NULL)
-    return;
-
-  link = (gchar *)g_hash_table_lookup(view->link_table,
-				      (gconstpointer)view->last_offset);
-
   view->last_offset = -1;
 
-  if (link == NULL)
-    return;
-
   gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(view),
 						 GTK_TEXT_WINDOW_TEXT),
 			view->ibeam_cursor);
@@ -780,8 +841,7 @@ emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 		bbs_thread_view_signals[LINK_MOUSE_OUT_SIGNAL],
 		0,
 		event,
-		view->thread,
-		link);
+		view->thread);
 }
 
 
@@ -801,18 +861,18 @@ bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
-      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
+      BBSThreadView *view = BBS_THREAD_VIEW(text_view);
       int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (bbs_thread_view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
+      gchar *link = (view->link_table != NULL)
+	? (gchar *)g_hash_table_lookup(view->link_table,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-      g_signal_emit(G_OBJECT(bbs_thread_view),
+      g_signal_emit(G_OBJECT(view),
 		    bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
 		    0,
 		    event,
-		    bbs_thread_view->thread,
+		    view->thread,
 		    link,
 		    &result);
     }
@@ -839,18 +899,18 @@ bbs_thread_view_button_release(GtkWidget *widget, GdkEventButton *event)
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
-      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
+      BBSThreadView *view = BBS_THREAD_VIEW(text_view);
       int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (bbs_thread_view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
+      gchar *link = (view->link_table != NULL)
+	? (gchar *)g_hash_table_lookup(view->link_table,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-      g_signal_emit(G_OBJECT(bbs_thread_view),
+      g_signal_emit(G_OBJECT(view),
 		    bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
 		    0,
 		    event,
-		    bbs_thread_view->thread,
+		    view->thread,
 		    link,
 		    &result);
     }
@@ -1006,15 +1066,18 @@ bbs_thread_view_append_text(BBSThreadView *view, const gchar *text, int len)
     }
 
   if (len != 0)
-    gtk_text_buffer_insert(view->text_buffer, &view->iter, text, len);
+    {
+      gtk_text_buffer_insert(view->text_buffer, &view->iter, text, len);
 
-  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &start, start_offset);
-  g_slist_foreach(view->tags, apply_tag, &args);
+      gtk_text_buffer_get_iter_at_offset(view->text_buffer,
+					 &start, start_offset);
+      g_slist_foreach(view->tags, apply_tag, &args);
 
-  if (text[len - 1] == '\n')
-    view->last_is_new_line = 1;
-  else
-    view->last_is_new_line = 0;
+      if (text[len - 1] == '\n')
+	view->last_is_new_line = 1;
+      else
+	view->last_is_new_line = 0;
+    }
 }
 
 
