@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.h,v 1.25 2004/01/19 00:38:47 fuyu Exp $
+ * $Id: bbs_thread_view.h,v 1.25.2.1 2004/05/04 13:52:30 fuyu Exp $
  */
 
 #ifndef _BBS_THREAD_VIEW_H_
@@ -124,8 +124,7 @@ struct _BBSThreadViewClass
 			  GdkEventMotion *event,
 			  OchushaBBSThread *thread, const gchar *link);
   void (*link_mouse_out)(BBSThreadView *bbs_thread_view,
-			 GdkEventAny *event,
-			 OchushaBBSThread *thread, const gchar *link);
+			 GdkEventAny *event, OchushaBBSThread *thread);
   gboolean (*link_mouse_press)(BBSThreadView *bbs_therad_view,
 			       GdkEventButton *event,
 			       OchushaBBSThread *thread, const gchar *link);
