@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.33 2004/01/27 15:49:11 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.35 2004/04/07 19:22:21 fuyu Exp $
  */
 
 #include "config.h"
@@ -35,6 +35,8 @@
 #include "boardlist_view.h"
 #include "marshal.h"
 
+#include "ugly_gtk2utils.h"
+
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -47,10 +49,26 @@
 
 
 static void boardlist_view_class_init(BoardlistViewClass *klass);
+#if GTK_MINOR_VERSION <= 2
 static void popup_menu_callback(gpointer data, guint action,
 				GtkWidget *widget);
 static void popup_menu_callback2(gpointer data, guint action,
 				 GtkWidget *widget);
+#else
+static void popup_menu_open_cb(GtkAction *action, GtkUIManager *manager);
+static void popup_menu_open_board_in_tab_cb(GtkAction *action,
+					    GtkUIManager *manager);
+static void popup_menu_open_board_with_web_browser_cb(GtkAction *action,
+						      GtkUIManager *manager);
+static void popup_menu_toggle_hidden_states_cb(GtkAction *action,
+					       GtkUIManager *manager);
+static void popup_menu_kill_items_cb(GtkAction *action, GtkUIManager *manager);
+static void popup_menu_copy_board_url_cb(GtkAction *action,
+					 GtkUIManager *manager);
+static void popup_menu_open_properties_cb(GtkAction *action,
+					  GtkUIManager *manager);
+#endif
+
 static void popup_menu_destructed_cb(gpointer data);
 static void boardlist_view_init(BoardlistView *view);
 
@@ -162,8 +180,13 @@ enum {
 static GtkTreeViewClass *parent_class = NULL;
 static int boardlist_view_signals[LAST_SIGNAL] =
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+
+#if GTK_MINOR_VERSION <= 2
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GtkItemFactory *popup_menu_item_factory2 = NULL;
+#else
+static GtkUIManager *ui_manager = NULL;
+#endif
 static GQuark node_info_id;
 
 static char *list_entry_color[4] = { NULL, NULL, NULL, NULL };
@@ -203,6 +226,7 @@ boardlist_view_class_init(BoardlistViewClass *klass)
   GtkBindingSet *binding_set;
 
   /* ݥåץåץ˥塼ν */
+#if GTK_MINOR_VERSION <= 2
   GtkItemFactoryEntry menu_items[] =
     {
       {
@@ -364,7 +388,107 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 						 NULL);
   gtk_item_factory_create_items(popup_menu_item_factory2,
 				nmenu_items2, menu_items2, NULL);
+#else
+  static GtkActionEntry popup_menu_entries[] =
+    {
+      {
+	"OpenBoardlistItems", GTK_STOCK_OPEN,
+	N_("_Open"), NULL,
+	N_("Open Board or Category"),
+	(GCallback)popup_menu_open_cb
+      },
+      {
+	"OpenBoardsInTab", GTK_STOCK_OPEN,
+	N_("_Open in _Tab"), NULL,
+	N_("Open Board in Tab"),
+	(GCallback)popup_menu_open_board_in_tab_cb
+      },
+      {
+	"OpenBoardsWithWebBrowser", GTK_STOCK_EXECUTE,
+	N_("Open with Web Browser"), NULL,
+	N_("Open Board with Web Browser"),
+	(GCallback)popup_menu_open_board_with_web_browser_cb
+      },
+      {
+	"ToggleBoardlistItemHiddenStates", NULL,
+	N_("Toggle Hide(_D)"), NULL,
+	N_("Toggle Hidden States of Items in Selection"),
+	(GCallback)popup_menu_toggle_hidden_states_cb
+      },
+      {
+	"HideBoardlistItems", NULL,
+	N_("Hide(_D)"), NULL,
+	N_("Hide Items in Selection"),
+	(GCallback)popup_menu_toggle_hidden_states_cb
+      },
+      {
+	"KillItems", GTK_STOCK_DELETE,
+	N_("_Kill"), NULL,
+	N_("Kill Items in Selection"),
+	(GCallback)popup_menu_kill_items_cb
+      },
+      {
+	"CopyBoardURL", GTK_STOCK_COPY,
+	N_("Copy Boards' URL"), NULL,
+	N_("Copy Boards' URL"),
+	(GCallback)popup_menu_copy_board_url_cb
+      },
+      {
+	"OpenProperties", GTK_STOCK_PROPERTIES,
+	N_("Open Properties"), NULL,
+	N_("Open Properties"),
+	(GCallback)popup_menu_open_properties_cb
+      }
+    };
 
+  static const char *popup_menu_ui =
+    "<ui>"
+    "  <popup name=\"BoardlistPopup\">"
+    "    <menuitem action=\"OpenBoardlistItems\"/>"
+    "    <menuitem action=\"OpenBoardsInTab\"/>"
+    "    <menuitem action=\"OpenBoardsWithWebBrowser\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"ToggleBoardlistItemHiddenStates\"/>"
+    "    <menuitem action=\"KillItems\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"CopyBoardURL\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"OpenProperties\"/>"
+    "  </popup>"
+    "  <popup name=\"SmartBoardlistPopup\">"
+    "    <menuitem action=\"OpenBoardlistItems\"/>"
+    "    <menuitem action=\"OpenBoardsInTab\"/>"
+    "    <menuitem action=\"OpenBoardsWithWebBrowser\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"HideBoardlistItems\"/>"
+    "    <menuitem action=\"KillItems\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"CopyBoardURL\"/>"
+    "    <separator/>"
+    "    <menuitem action=\"OpenProperties\"/>"
+    "  </popup>"
+    "</ui>";
+
+  GError *error = NULL;
+  GtkActionGroup *action_group
+    = gtk_action_group_new("BoardlistViewPopupActions");
+  ui_manager = gtk_ui_manager_new();
+
+  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
+  gtk_action_group_add_actions(action_group,
+			       popup_menu_entries,
+			       G_N_ELEMENTS(popup_menu_entries),
+			       ui_manager);
+  gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
+
+  if (!gtk_ui_manager_add_ui_from_string(ui_manager, popup_menu_ui, -1,
+					 &error))
+    {
+      g_message("building menus failed: %s\n", error->message);
+      g_error_free(error);
+      exit(EXIT_FAILURE);
+    }
+#endif
 
   parent_class = g_type_class_peek_parent(klass);
   binding_set = gtk_binding_set_by_class(klass);
@@ -1342,6 +1466,7 @@ boardlist_view_do_on_item_at_cursor(BoardlistView *view, int what_to_do)
 }
 
 
+#if GTK_MINOR_VERSION <= 2
 static void
 popup_menu_callback(gpointer data, guint action, GtkWidget *widget)
 {
@@ -1366,6 +1491,63 @@ popup_menu_callback2(gpointer data, guint action, GtkWidget *widget)
 #endif
   boardlist_view_do_on_selection(view, action);
 }
+#else
+static void
+popup_menu_open_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_OPEN_SELECTION);
+}
+
+
+static void
+popup_menu_open_board_in_tab_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_OPEN_SELECTION_IN_TAB);
+}
+
+
+static void
+popup_menu_open_board_with_web_browser_cb(GtkAction *action,
+					  GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_OPEN_SELECTION_WITH_BROWSER);
+}
+
+
+static void
+popup_menu_toggle_hidden_states_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_TOGGLE_HIDE_SELECTION);
+}
+
+
+static void
+popup_menu_kill_items_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_KILL_SELECTION);
+}
+
+
+static void
+popup_menu_copy_board_url_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_COPY_BOARD_URL);
+}
+
+
+static void
+popup_menu_open_properties_cb(GtkAction *action, GtkUIManager *manager)
+{
+  BoardlistView *view = ochusha_ui_manager_get_popup_data(manager);
+  boardlist_view_do_on_selection(view, POPUP_MENU_OPEN_PROPERTIES);
+}
+#endif
 
 
 static void
@@ -1411,6 +1593,7 @@ boardlist_view_button_press(GtkWidget *widget, GdkEventButton *event)
 
   if (event->button == 3)
     {
+#if GTK_MINOR_VERSION <= 2
       if (view->hide_hidden_items)
 	gtk_item_factory_popup_with_data(popup_menu_item_factory2,
 					 view, popup_menu_destructed_cb,
@@ -1423,6 +1606,22 @@ boardlist_view_button_press(GtkWidget *widget, GdkEventButton *event)
 					 event->x_root, event->y_root,
 					 event->button,
 					 gtk_get_current_event_time());
+#else
+      if (view->hide_hidden_items)
+	ochusha_ui_manager_popup_with_data(ui_manager,
+					   "/SmartBoardlistPopup",
+					   view, popup_menu_destructed_cb,
+					   event->x_root, event->y_root,
+					   event->button,
+					   gtk_get_current_event_time());
+      else
+	ochusha_ui_manager_popup_with_data(ui_manager,
+					   "/BoardlistPopup",
+					   view, popup_menu_destructed_cb,
+					   event->x_root, event->y_root,
+					   event->button,
+					   gtk_get_current_event_time());
+#endif
 
       if (!gtk_tree_selection_iter_is_selected(view->selection, &iter))
 	{
