@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: download_ui.c,v 1.11 2004/01/27 11:56:11 fuyu Exp $
+ * $Id: download_ui.c,v 1.11.2.1 2004/05/01 04:34:27 fuyu Exp $
  */
 
 #include "config.h"
@@ -220,6 +220,8 @@ download_dialog_response_cb(GtkWidget *widget, int response_id,
 
  done:
   OCHU_OBJECT_UNREF(buffer);
+  gtk_widget_hide(widget);
+  gtk_widget_unrealize(widget);
   gtk_widget_destroy(widget);
   ENTER_MAIN_EVENT_LOOP();
 }
@@ -259,8 +261,14 @@ filesel_dialog_response_cb(GtkWidget *widget, int response_id,
 
   MAIN_THREAD_EVENT_CALLBACK();
   if (response_id != GTK_RESPONSE_OK)
-    gtk_widget_destroy(download_dialog);
+    {
+      gtk_widget_hide(download_dialog);
+      gtk_widget_unrealize(download_dialog);
+      gtk_widget_destroy(download_dialog);
+    }
 
+  gtk_widget_hide(widget);
+  gtk_widget_unrealize(widget);
   gtk_widget_destroy(widget);
   ENTER_MAIN_EVENT_LOOP();
 }
