@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: main.c,v 1.28 2004/02/27 22:19:59 fuyu Exp $
+ * $Id: main.c,v 1.28.2.1 2004/05/01 04:34:27 fuyu Exp $
  */
 
 #include "config.h"
@@ -81,7 +81,11 @@ real_main(OchushaApplication *application)
 
 #if TRACE_MEMORY_USAGE
   if (application->top_level != NULL)
-    gtk_widget_destroy(GTK_WIDGET(application->top_level));
+    {
+      gtk_widget_hide(GTK_WIDGET(application->top_level));
+      gtk_widget_unrealize(GTK_WIDGET(application->top_level));
+      gtk_widget_destroy(GTK_WIDGET(application->top_level));
+    }
 #endif
 
 #if 1
@@ -299,7 +303,11 @@ main(int argc, char *argv[])
 
 # if TRACE_MEMORY_USAGE
   if (application->top_level != NULL)
-    gtk_widget_destroy(GTK_WIDGET(application->top_level));
+    {
+      gtk_widget_hide(GTK_WIDGET(application->top_level));
+      gtk_widget_unrealize(GTK_WIDGET(application->top_level));
+      gtk_widget_destroy(GTK_WIDGET(application->top_level));
+    }
 # endif
 
 # if 1
