@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: regex_utils.c,v 1.6 2004/01/14 10:03:18 fuyu Exp $
+ * $Id: regex_utils.c,v 1.7 2004/03/21 21:35:11 fuyu Exp $
  */
 
 #include "config.h"
@@ -41,6 +41,10 @@
 # include <oniguruma.h>
 #endif
 
+#if (ONIGURUMA_VERSION_MAJOR >= 2) && (ONIGURUMA_VERSION_MINOR > 0)
+# define ONIGURUMA_2_2_API		1
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -134,7 +138,11 @@ regex_editor_finalize(GObject *object)
 
   if (editor->expression != NULL)
     {
+#ifdef ONIGURUMA_2_2_API
+      onig_free((regex_t *)editor->expression);
+#else
       regex_free((regex_t *)editor->expression);
+#endif
       editor->expression = NULL;
     }
   if (editor->regex_buffer != NULL)
@@ -185,13 +193,19 @@ regex_editor_synthesize_regular_expression(const gchar *text)
       if (tmp_pos == end_pos)
 	goto next_line;
 
+#ifdef ONIGURUMA_2_2_API
+      if (onig_new(&regex, tmp_pos, end_pos, ONIG_OPTION_EXTEND,
+		   ONIG_ENCODING_UTF8,
+		   ONIG_SYNTAX_POSIX_EXTENDED, NULL) == ONIG_NORMAL)
+#else
       if (regex_new(&regex, tmp_pos, end_pos, REG_OPTION_EXTEND,
-#ifdef REG_ENCODING_UTF8
+# ifdef REG_ENCODING_UTF8
 		    REG_ENCODING_UTF8,
-#else /* old style */
+# else /* old style */
 		    REGCODE_UTF8,
-#endif
+# endif
 		    REG_SYNTAX_POSIX_EXTENDED, NULL) == REG_NORMAL)
+#endif
 	{
 	  gchar *sub_exp = G_STRNDUP(tmp_pos, end_pos - tmp_pos + 1);
 	  if (result == NULL)
@@ -206,7 +220,11 @@ regex_editor_synthesize_regular_expression(const gchar *text)
 	    }
 	}
 
+#ifdef ONIGURUMA_2_2_API
+      onig_free(regex);
+#else
       regex_free(regex);
+#endif
 
     next_line:
       tmp_pos = end_pos + 1;
@@ -311,13 +329,32 @@ gpointer
 oniguruma_regex_new(const gchar *pattern, const char *encoding)
 {
   regex_t *result;
+#ifdef ONIGURUMA_2_2_API
+  OnigEncoding code;
+#else
   RegCharEncoding code;
+#endif
 
   g_return_val_if_fail(pattern != NULL && *pattern != '\0', NULL);
 
   if (encoding != NULL)
     {
-#ifdef REG_ENCODING_UNDEF
+#ifdef ONIGURUMA_2_2_API
+      if (strcmp(encoding, "CP932") == 0)
+	code = ONIG_ENCODING_SJIS;
+      else if (strcmp(encoding, "EUC-JP") == 0)
+	code = ONIG_ENCODING_EUC_JP;
+      else if (strcmp(encoding, "UTF-8") == 0)
+	code = ONIG_ENCODING_UTF8;
+      else if (strcmp(encoding, "Shift_JIS") == 0)
+	code = ONIG_ENCODING_SJIS;
+      else
+	code = ONIG_ENCODING_UTF8;
+    }
+  else
+    code = ONIG_ENCODING_UTF8;
+#else
+# ifdef REG_ENCODING_UNDEF
       if (strcmp(encoding, "CP932") == 0)
 	code = REG_ENCODING_SJIS;
       else if (strcmp(encoding, "EUC-JP") == 0)
@@ -331,7 +368,7 @@ oniguruma_regex_new(const gchar *pattern, const char *encoding)
     }
   else
     code = REG_ENCODING_UTF8;
-#else /* old style */
+# else /* old style */
       if (strcmp(encoding, "CP932") == 0)
 	code = REGCODE_SJIS;
       else if (strcmp(encoding, "EUC-JP") == 0)
@@ -345,8 +382,19 @@ oniguruma_regex_new(const gchar *pattern, const char *encoding)
     }
   else
     code = REGCODE_UTF8;
+# endif
 #endif
 
+#ifdef ONIGURUMA_2_2_API
+  if (onig_new(&result, (UChar *)pattern, (UChar *)pattern + strlen(pattern),
+	       ONIG_OPTION_EXTEND, code,
+	       ONIG_SYNTAX_POSIX_EXTENDED, NULL) != ONIG_NORMAL)
+    {
+      onig_free(result);
+      fprintf(stderr, "Invalid regular expression: \"%s\"\n", pattern);
+      return NULL;
+    }
+#else
   if (regex_new(&result, (UChar *)pattern, (UChar *)pattern + strlen(pattern),
 		REG_OPTION_EXTEND, code,
 		REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
@@ -355,6 +403,7 @@ oniguruma_regex_new(const gchar *pattern, const char *encoding)
       fprintf(stderr, "Invalid regular expression: \"%s\"\n", pattern);
       return NULL;
     }
+#endif
 
   return result;
 }
@@ -365,7 +414,11 @@ oniguruma_regex_free(gpointer oniguruma_regex)
 {
   g_return_if_fail(oniguruma_regex != NULL);
 
+#ifdef ONIGURUMA_2_2_API
+  onig_free((regex_t *)oniguruma_regex);
+#else
   regex_free((regex_t *)oniguruma_regex);
+#endif
 }
 
 
@@ -379,7 +432,13 @@ oniguruma_regex_match(gpointer oniguruma_regex, const char *text)
 
   text_end = (UChar *)text + strlen(text);
 
+#ifdef ONIGURUMA_2_2_API
+  return onig_search((regex_t *)oniguruma_regex,
+		     (UChar *)text, text_end,
+		     (UChar *)text, text_end, NULL, 0) >= 0;
+#else
   return regex_search((regex_t *)oniguruma_regex,
 		      (UChar *)text, text_end,
 		      (UChar *)text, text_end, NULL, 0) >= 0;
+#endif
 }
