@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: response_editor.h,v 1.7 2004/02/28 22:59:25 fuyu Exp $
+ * $Id: response_editor.h,v 1.8.2.1 2004/05/03 06:58:34 fuyu Exp $
  */
 
 #ifndef _RESPONSE_EDITOR_H_
@@ -57,8 +57,13 @@ struct _ResponseEditor
   GtkBox *hbox;
 
   /* hboxȤname_combomail_comboȤդ٥ */
+#if GTK_MINOR_VERSION <= 2
   GtkCombo *name_combo;
   GtkCombo *mail_combo;
+#else
+  GtkComboBox *name_combo;
+  GtkComboBox *mail_combo;
+#endif
 
   GtkTextView *text_view;
 
@@ -89,6 +94,8 @@ void response_editor_add_mail(ResponseEditor *editor, const gchar *name);
 
 void response_editor_clear_response(ResponseEditor *editor);
 gchar *response_editor_get_response(ResponseEditor *editor);
+void response_editor_append_response(ResponseEditor *editor,
+				     const gchar *response);
 #if 0
 void response_editor_set_response(ResponseEditor *editor,
 				  const gchar *response);
