@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.13 2004/01/25 16:29:37 fuyu Exp $
+ * $Id: htmlutils.c,v 1.14 2004/02/27 23:25:34 fuyu Exp $
  */
 
 #include "config.h"
@@ -744,13 +744,17 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
   int n_attrs = 0;
   char *attrs[MAX_ATTRIBUTES * 2 + 1];
   char *name;
-  const char *cur_pos = ++head;
+  const char *cur_pos = head + 1;
   const char *tmp_pos = mempbrk(cur_pos, " \t\n>", --len);
   gboolean end_tag;
   gboolean buffer_full = FALSE;
 
   if (tmp_pos == NULL)
-    return NULL;
+    {
+      name = NULL;
+      cur_pos = NULL;
+      goto finish;
+    }
 
   if (*cur_pos == '/')
     {
@@ -854,6 +858,20 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
 
   attrs[n_attrs * 2] = NULL;
 
+  if (name != NULL)
+    {
+      tmp_pos = name;
+      while (*tmp_pos != '\0')
+	{
+	  if (!g_ascii_isalnum(*tmp_pos))
+	    {
+	      cur_pos = NULL;
+	      goto finish;
+	    }
+	  tmp_pos++;
+	}
+    }
+
   if (end_tag)
     {
       if (handler->endElement != NULL)
@@ -870,7 +888,14 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
     }
 
  finish:
-  G_FREE(name);
+  if (cur_pos == NULL)
+    {
+      cur_pos = head;
+      if (handler->characters != NULL)
+	(*handler->characters)(context, cur_pos, 1);
+    }
+  if (name != NULL)
+    G_FREE(name);
 
   while (--n_attrs >= 0)
     {
@@ -879,7 +904,7 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
 	G_FREE(attrs[n_attrs * 2 + 1]);
     }
 
-  return (cur_pos != NULL) ? cur_pos + 1 : NULL;
+  return cur_pos + 1;
 }
 
 
