@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.35 2004/02/26 06:05:10 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.37 2004/04/13 15:38:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -70,7 +70,7 @@ static void ochusha_bulletin_board_read_boardlist_element(
 						GHashTable *board_attributes);
 static void ochusha_bulletin_board_write_boardlist_element(
 						OchushaBulletinBoard *board,
-						gzFile *boardlist_xml);
+						gzFile boardlist_xml);
 
 GType
 ochusha_bulletin_board_get_type(void)
@@ -201,6 +201,7 @@ ochusha_bulletin_board_class_init(OchushaBulletinBoardClass *klass)
   klass->recover_threadlist = NULL;
   klass->generate_base_path = NULL;
   klass->generate_board_id = ochusha_bulletin_board_generate_board_id;
+  klass->server_changed = NULL;
   klass->thread_new = NULL;
   klass->lookup_thread_by_url = NULL;
   klass->lookup_kako_thread_by_url = NULL;
@@ -400,6 +401,7 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
 				    const char *url)
 {
   OchushaBulletinBoardClass *klass;
+  char *prev_server;
   char *server;
   char *base_path;
 
@@ -412,12 +414,18 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
     G_FREE(board->base_url);
   board->base_url = G_STRDUP(url);
 
-  if (board->server != NULL)
-    G_FREE(board->server);
+  prev_server = board->server;
   board->server = server;
 
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
 
+  if (prev_server != NULL && strcmp(prev_server, server) != 0
+      && *klass->server_changed != NULL)
+    (*klass->server_changed)(board, prev_server);
+
+  if (prev_server != NULL)
+    G_FREE(prev_server);
+
   if (*klass->generate_base_path)
     base_path = (*klass->generate_base_path)(board, url);
   else
@@ -871,7 +879,7 @@ ochusha_bulletin_board_read_boardlist_element(OchushaBulletinBoard *board,
 
 static void
 ochusha_bulletin_board_write_boardlist_element(OchushaBulletinBoard *board,
-					       gzFile *boardlist_xml)
+					       gzFile boardlist_xml)
 {
   OUTPUT_BOARD_ATTRIBUTE_INT(boardlist_xml, board, bbs_type);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, board, name);
@@ -1341,7 +1349,7 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 typedef struct _WriteThreadlistArgs
 {
   OchushaBulletinBoard *board;
-  gzFile *threadlist_xml;
+  gzFile threadlist_xml;
 } WriteThreadlistArgs;
 
 
@@ -1350,7 +1358,7 @@ write_bbs_thread(gpointer data, gpointer user_data)
 {
   OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(data);
   WriteThreadlistArgs *args = (WriteThreadlistArgs *)user_data;
-  gzFile *threadlist_xml = args->threadlist_xml;
+  gzFile threadlist_xml = args->threadlist_xml;
 
   gzprintf(threadlist_xml, "    <thread>\n");
 
@@ -1375,7 +1383,7 @@ ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
   WriteThreadlistArgs args;
   char pathname[PATH_MAX];
   int fd;
-  gzFile *threadlist_xml;
+  gzFile threadlist_xml;
 
   g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board) && config != NULL,
 		       FALSE);
