@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.60.2.1 2004/05/01 16:02:58 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.60.2.2 2004/05/04 03:05:09 fuyu Exp $
  */
 
 #include "config.h"
@@ -2090,8 +2090,17 @@ ochusha_network_broker_read_from_url_full(OchushaNetworkBroker *broker,
 	return buffer;
 
       if (mode == OCHUSHA_NETWORK_BROKER_IMAGE_CACHE)
-	fd = ochusha_config_image_cache_open_file(broker->config,
-						  cache_url, O_RDONLY);
+	{
+	  fd = ochusha_config_image_cache_open_file(broker->config,
+						    cache_url, O_RDONLY);
+	  if (fd < 0)
+	    {
+	      char *cache_name = ochusha_utils_url_encode_string(cache_url);
+	      fd = ochusha_config_open_file(broker->config, cache_name,
+					    "image_cache", O_RDONLY);
+	      G_FREE(cache_name);
+	    }
+	}
       else
 	fd = ochusha_config_cache_open_file(broker->config,
 					    cache_url, O_RDONLY);
