@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.69 2004/02/27 22:17:12 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.71 2004/04/13 15:38:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -68,7 +68,7 @@ static void ochusha_thread_2ch_read_threadlist_element(
 						GHashTable *thread_attributes);
 static void ochusha_thread_2ch_write_threadlist_element(
 						OchushaBBSThread *thread,
-						gzFile *threadlist_xml);
+						gzFile threadlist_xml);
 
 static void ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread);
 
@@ -326,7 +326,7 @@ ochusha_thread_2ch_read_threadlist_element(OchushaBBSThread *thread,
 
 static void
 ochusha_thread_2ch_write_threadlist_element(OchushaBBSThread *thread,
-					    gzFile *threadlist_xml)
+					    gzFile threadlist_xml)
 {
   OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
@@ -723,6 +723,8 @@ process_offlaw_result(WorkerThread *employee, ProcessOfflawResultJobArgs *args)
       else
 	{
 	  /* Υ顼(¿ʬåIDδڤ) */
+	  OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
+	  char *old_url;
 	  const char *encoding
 	    = ochusha_bbs_thread_get_response_character_encoding(thread);
 	  iconv_helper *helper
@@ -747,6 +749,11 @@ process_offlaw_result(WorkerThread *employee, ProcessOfflawResultJobArgs *args)
 	  ochusha_async_buffer_unlock(cgi_buffer);
 	  ochusha_async_buffer_active_unref(cgi_buffer);
 	  ochusha_async_buffer_active_unref(dat_buffer);
+
+	  old_url = thread_2ch->oyster_url;
+	  thread_2ch->oyster_url = NULL;
+	  G_FREE(old_url);
+
 	  goto terminated;
 	}
     }
@@ -870,8 +877,9 @@ ochusha_thread_2ch_read_dat_by_offlaw(OchushaThread2ch *thread_2ch,
   OchushaAsyncBuffer *dat_buffer = buffer;
   OchushaAsyncBuffer *cgi_buffer;
   OchushaNetworkBrokerBufferStatus *status;
-  const char *url = ochusha_thread_2ch_get_oyster_url(thread_2ch,
-						      broker->config->session_id_2ch);
+  const char *url
+    = ochusha_thread_2ch_get_oyster_url(thread_2ch,
+					broker->config->session_id_2ch);
 
   if (dat_buffer != NULL && !ochusha_async_buffer_reset(dat_buffer))
     {
