@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.37 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.40 2004/02/20 09:12:48 fuyu Exp $
  */
 
 #include "config.h"
@@ -817,6 +817,9 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
 	    }
 	}
 
+      if (strstr(url, "bbq.2ch.net") != NULL)
+	board->bbs_type = OCHUSHA_BBS_TYPE_2CH_HEADLINE;
+
       board->killed = FALSE;
       ochusha_bulletin_board_set_name(board, name);
       ochusha_board_category_add_board(category, board);
@@ -1087,6 +1090,15 @@ ochusha_utils_2ch_try_post(OchushaNetworkBroker *broker,
 	    { /* "åǧ" in Shift_JISå */
 	      if (strstr(post_status.body, "\203\116\203\142\203\114\201\133\212\155\224\106\201\111") != NULL)
 		x_2ch = "2ch_X:cookie";	/*  */
+	      else if (strstr(post_status.body, "\223\212\215\145\212\155\224\106") != NULL)
+		{
+		  /* "Ƴǧ" in Shift_JISå */
+		  x_2ch = "2ch_X:cookie";
+		  /* äξ硢submit="񤭹"Ǥʤ
+		   * submit="Ǥ餦Ȥƽ񤭹"
+		   * ǤʤȤᤫ
+		   */
+		}
 	    }
 	  if (x_2ch != NULL)
 	    {
@@ -1198,3 +1210,114 @@ ochusha_utils_2ch_try_post(OchushaNetworkBroker *broker,
   return post_result
     ? OCHUSHA_UTILS_2CH_POST_SUCCESS : OCHUSHA_UTILS_2CH_POST_FAILURE;
 }
+
+
+void
+ochusha_utils_2ch_login(OchushaConfig *config, OchushaNetworkBroker *broker,
+			const char *user_id, const char *passwd)
+{
+  char content_body[4096];
+  const char *headers[] =
+    {
+      "User-Agent", OCHUSHA_DOLIB_USER_AGENT,
+      "X-2ch-UA", OCHUSHA_DOLIB_X_2CH_UA,
+      NULL,
+    };
+  OchushaNetworkBrokerPostStatus post_status;
+  char *session_id = NULL;
+  gboolean post_result;
+
+  config->login_2ch = FALSE;
+  if (config->session_id_2ch != NULL)
+    {
+      G_FREE(config->session_id_2ch);
+      config->session_id_2ch = NULL;
+    }
+
+  g_return_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker));
+  g_return_if_fail(user_id != NULL && passwd != NULL);
+
+  snprintf(content_body, 4096, "ID=%s&PW=%s", user_id, passwd);
+  post_result = ochusha_network_broker_try_post_raw(broker,
+						    OCHUSHA_DOLIB_URL,
+						    OCHUSHA_DOLIB_HOST,
+						    headers, content_body,
+						    &post_status);
+
+  if (post_result)
+    {
+#if DEBUG_LOGIN
+      gchar message[4096];
+      snprintf(message, 4096, _("Status: %d\n"), post_status.status_code);
+      ochusha_network_broker_output_log(broker, message);
+
+      if (post_status.body != NULL)
+	{
+	  iconv_t converter = iconv_open("UTF-8//IGNORE", "CP932");
+	  if (converter != (iconv_t)-1)
+	    {
+	      gchar *body_utf8 = convert_string(converter, NULL,
+						post_status.body, -1);
+	      if (body_utf8 != NULL)
+		{
+		  ochusha_network_broker_output_log(broker,
+						    "Result-Body: ");
+		  ochusha_network_broker_output_log(broker, body_utf8);
+		  ochusha_network_broker_output_log(broker, "\n");
+		  G_FREE(body_utf8);
+		}
+	      else
+		{
+		  ochusha_network_broker_output_log(broker,
+						    "iconv_failed.\n");
+		}
+	      iconv_close(converter);
+	    }
+	  else
+	    {
+	      ochusha_network_broker_output_log(broker,
+						"iconv_open failed.\n");
+	    }
+	}
+#endif
+
+      if (post_status.status_code == 200)
+	{
+	  const char *sid = strstr(post_status.body, "SESSION-ID=");
+
+	  if (sid != NULL)
+	    {
+	      sid += 11;	/* "SESSION-ID="ɤФ */
+	      if (strncmp(sid, "ERROR:", 6) != 0)
+		{
+		  char *eol_pos = strchr(sid, '\n');
+		  if (eol_pos != NULL)
+		    *eol_pos = '\0';
+		  session_id = ochusha_utils_url_encode_string(sid);
+		}
+	    }
+	}
+    }
+
+  if (post_status.body != NULL)
+    G_FREE(post_status.body);
+
+  if (post_status.set_cookie != NULL)
+    G_FREE(post_status.set_cookie);
+
+  config->session_id_2ch = session_id;
+  if (session_id != NULL)
+    config->login_2ch = TRUE;
+}
+
+
+void
+ochusha_utils_2ch_logout(OchushaConfig *config)
+{
+  config->login_2ch = FALSE;
+  if (config->session_id_2ch != NULL)
+    {
+      G_FREE(config->session_id_2ch);
+      config->session_id_2ch = NULL;
+    }
+}
