@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.26 2004/01/20 07:48:43 fuyu Exp $
+ * $Id: utils.c,v 1.30 2004/04/13 15:36:12 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION	1
@@ -46,7 +46,7 @@
 #include <fcntl.h>
 #include <limits.h>
 
-#if TRACE_MEMORY_USAGE || TRACE_REFCOUNT
+#if !defined(HAVE_TIMEGM) || TRACE_MEMORY_USAGE || TRACE_REFCOUNT
 #include <pthread.h>
 #endif
 
@@ -494,11 +494,13 @@ ochusha_utils_get_utc_time(const char *date_string)
 {
   static char *lc_time = NULL;
   struct tm tm;
+  char *parse_result;
 
   if (lc_time == NULL)
     {
 #if HAVE_LOCALE_H
       lc_time = setlocale(LC_TIME, "C");
+      tzset();
       g_return_val_if_fail(lc_time != NULL, -1);
 #else
       return -1;
@@ -506,9 +508,34 @@ ochusha_utils_get_utc_time(const char *date_string)
     }
 
   g_return_val_if_fail(date_string != NULL && *date_string != '\0', -1);
-  g_return_val_if_fail(strptime(date_string, "%a, %d %b %Y %T %Z", &tm)!= NULL,
-		       -1);
+
+  memset(&tm, 0, sizeof(struct tm));
+#if 0
+  /*
+   * GMTΤ褦timezoneλꤵ줿date_string顢
+   * struct tmǤĤäΤ
+   * LinuxǤtimezone̵꤬뤵Ƥ͡
+   * SolarisǤ%Z"GMT"ȥ顼ˤʤäƤޤ
+   */
+  parse_result = strptime(date_string, "%a, %d %b %Y %T %Z", &tm);
+  g_return_val_if_fail(parse_result != NULL, -1);
+
   return mktime(&tm);
+#else
+  /*
+   * ȡdate_string"GMT"ʳ褿˻郎
+   * 2chΥФ˴ؤ¤Linux/SolarisǤԤ褦
+   * 郎֤äƤΤǡ괺Ƥ
+   */
+  parse_result = strptime(date_string, "%a, %d %b %Y %T", &tm);
+  g_return_val_if_fail(parse_result != NULL, -1);
+
+# ifdef HAVE_TIMEGM
+  return timegm(&tm);
+# else /* Ķad-hoc塼 */
+  return mktime(&tm) + (strstr(date_string, "GMT") != NULL ? 32400 : 0);
+# endif
+#endif
 }
 
 
