@@ -2,41 +2,43 @@
 
   regint.h -  Oniguruma (regular expression library)
 
-  Copyright (C) 2002-2003  K.Kosako (kosako@sofnec.co.jp)
+  Copyright (C) 2002-2004  K.Kosako (kosako@sofnec.co.jp)
 
 **********************************************************************/
 #ifndef REGINT_H
 #define REGINT_H
 
 /* for debug */
-/* #define REG_DEBUG_PARSE_TREE */
-/* #define REG_DEBUG_COMPILE */
-/* #define REG_DEBUG_SEARCH */
-/* #define REG_DEBUG_MATCH */
-/* #define REG_DONT_OPTIMIZE */
+/* #define ONIG_DEBUG_PARSE_TREE */
+/* #define ONIG_DEBUG_COMPILE */
+/* #define ONIG_DEBUG_SEARCH */
+/* #define ONIG_DEBUG_MATCH */
+/* #define ONIG_DONT_OPTIMIZE */
 
 /* for byte-code statistical data. */
-/* #define REG_DEBUG_STATISTICS */
+/* #define ONIG_DEBUG_STATISTICS */
 
-#if defined(REG_DEBUG_PARSE_TREE) || defined(REG_DEBUG_MATCH) || \
-    defined(REG_DEBUG_COMPILE) || defined(REG_DEBUG_STATISTICS)
-#ifndef REG_DEBUG
-#define REG_DEBUG
+#if defined(ONIG_DEBUG_PARSE_TREE) || defined(ONIG_DEBUG_MATCH) || \
+    defined(ONIG_DEBUG_COMPILE) || defined(ONIG_DEBUG_STATISTICS)
+#ifndef ONIG_DEBUG
+#define ONIG_DEBUG
 #endif
 #endif
 
 #if defined(__i386) || defined(__i386__) || defined(_M_IX86) || \
     (defined(__ppc__) && defined(__APPLE__)) || \
-    defined(__mc68020__) || defined(__x86_64) || defined(__x86_64__)
-#define UNALIGNED_WORD_ACCESS
+    defined(__x86_64) || defined(__x86_64__) || \
+    defined(__mc68020__)
+#define PLATFORM_UNALIGNED_WORD_ACCESS
 #endif
 
 /* config */
 /* spec. config */
 #define USE_NAMED_GROUP
 #define USE_SUBEXP_CALL
-#define USE_FOLD_MATCH                                /* ess-tsett etc... */ 
-#define USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE   /* /\n$/ =~ "\n" */
+#define USE_FOLD_MATCH                                  /* ess-tsett etc... */
+#define USE_INFINITE_REPEAT_MONOMANIAC_MEM_STATUS_CHECK /* /(?:()|())*\2/ */
+#define USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE     /* /\n$/ =~ "\n" */
 #define USE_WARNING_REDUNDANT_NESTED_REPEAT_OPERATOR
 /* internal config */
 #define USE_RECYCLE_NODE
@@ -44,7 +46,7 @@
 #define USE_QUALIFIER_PEEK_NEXT
 
 #define INIT_MATCH_STACK_SIZE                     160
-#define MATCH_STACK_LIMIT_SIZE                 200000
+#define MATCH_STACK_LIMIT_SIZE                 500000
 
 /* interface to external system */
 #ifdef NOT_RUBY      /* gived from Makefile */
@@ -83,7 +85,7 @@
 #define xmemset     memset
 #define xmemcpy     memcpy
 #define xmemmove    memmove
-#if defined(_WIN32) && !defined(__CYGWIN__)
+#if defined(_WIN32) && !defined(__GNUC__)
 #define xalloca     _alloca
 #ifdef NOT_RUBY
 #define vsnprintf   _vsnprintf
@@ -109,17 +111,12 @@
 #include <ctype.h>
 #include <sys/types.h>
 
-#ifdef REG_DEBUG
+#ifdef ONIG_DEBUG
 # include <stdio.h>
 #endif
 
-#ifdef NOT_RUBY
-# include "oniguruma.h"
-#else
-# include "regex.h"
-#endif
-
 #include "regenc.h"
+#include "oniguruma.h"
 
 #ifdef MIN
 #undef MIN
@@ -130,17 +127,24 @@
 #define MIN(a,b) (((a)>(b))?(b):(a))
 #define MAX(a,b) (((a)<(b))?(b):(a))
 
-#ifndef UNALIGNED_WORD_ACCESS
+#define IS_NULL(p)                    (((void*)(p)) == (void*)0)
+#define IS_NOT_NULL(p)                (((void*)(p)) != (void*)0)
+#define CHECK_NULL_RETURN(p)          if (IS_NULL(p)) return NULL
+#define CHECK_NULL_RETURN_VAL(p,val)  if (IS_NULL(p)) return (val)
+#define NULL_UCHARP                   ((UChar* )0)
+
+#ifndef PLATFORM_UNALIGNED_WORD_ACCESS
 #define WORD_ALIGNMENT_SIZE       SIZEOF_INT
 
 #define GET_ALIGNMENT_PAD_SIZE(addr,pad_size) do {\
-  (pad_size) = WORD_ALIGNMENT_SIZE - ((int )(addr) % WORD_ALIGNMENT_SIZE);\
+  (pad_size) = WORD_ALIGNMENT_SIZE \
+               - ((unsigned int )(addr) % WORD_ALIGNMENT_SIZE);\
   if ((pad_size) == WORD_ALIGNMENT_SIZE) (pad_size) = 0;\
 } while (0)
 
 #define ALIGNMENT_RIGHT(addr) do {\
   (addr) += (WORD_ALIGNMENT_SIZE - 1);\
-  (addr) -= ((int )(addr) % WORD_ALIGNMENT_SIZE);\
+  (addr) -= ((unsigned int )(addr) % WORD_ALIGNMENT_SIZE);\
 } while (0)
 
 
@@ -223,7 +227,7 @@
 #define SERIALIZE_UINT(i,p)    SERIALIZE_8BYTE_INT(i,p)
 #endif
 
-#endif /* UNALIGNED_WORD_ACCESS */
+#endif /* PLATFORM_UNALIGNED_WORD_ACCESS */
 
 /* stack pop level */
 #define STACK_POP_LEVEL_FREE        0
@@ -231,12 +235,12 @@
 #define STACK_POP_LEVEL_ALL         2
 
 /* optimize flags */
-#define REG_OPTIMIZE_NONE              0
-#define REG_OPTIMIZE_EXACT             1   /* Slow Search */
-#define REG_OPTIMIZE_EXACT_BM          2   /* Boyer Moore Search */
-#define REG_OPTIMIZE_EXACT_BM_NOT_REV  3   /* BM   (but not simple match) */
-#define REG_OPTIMIZE_EXACT_IC          4   /* Slow Search (ignore case) */
-#define REG_OPTIMIZE_MAP               5   /* char map */
+#define ONIG_OPTIMIZE_NONE              0
+#define ONIG_OPTIMIZE_EXACT             1   /* Slow Search */
+#define ONIG_OPTIMIZE_EXACT_BM          2   /* Boyer Moore Search */
+#define ONIG_OPTIMIZE_EXACT_BM_NOT_REV  3   /* BM   (but not simple match) */
+#define ONIG_OPTIMIZE_EXACT_IC          4   /* Slow Search (ignore case) */
+#define ONIG_OPTIMIZE_MAP               5   /* char map */
 
 /* bit status */
 typedef unsigned int  BitStatusType;
@@ -262,54 +266,40 @@ typedef unsigned int  BitStatusType;
 
 #define INT_MAX_LIMIT           ((1UL << (SIZEOF_INT * 8 - 1)) - 1)
 
-#define INFINITE_DISTANCE  ~((RegDistance )0)
-
 #define DIGITVAL(code)    ((code) - '0')
 #define ODIGITVAL(code)   DIGITVAL(code)
 #define XDIGITVAL(enc,code) \
-  (ENC_IS_CODE_DIGIT(enc,code) ? DIGITVAL(code) \
-       : (ENC_IS_CODE_UPPER(enc,code) ? (code) - 'A' + 10 : (code) - 'a' + 10))
+  (ONIGENC_IS_CODE_DIGIT(enc,code) ? DIGITVAL(code) \
+   : (ONIGENC_IS_CODE_UPPER(enc,code) ? (code) - 'A' + 10 : (code) - 'a' + 10))
 
-#define IS_SINGLELINE(option)     ((option) & REG_OPTION_SINGLELINE)
-#define IS_MULTILINE(option)      ((option) & REG_OPTION_MULTILINE)
-#define IS_IGNORECASE(option)     ((option) & REG_OPTION_IGNORECASE)
-#define IS_EXTEND(option)         ((option) & REG_OPTION_EXTEND)
-#define IS_FIND_LONGEST(option)   ((option) & REG_OPTION_FIND_LONGEST)
-#define IS_FIND_NOT_EMPTY(option) ((option) & REG_OPTION_FIND_NOT_EMPTY)
+#define IS_SINGLELINE(option)     ((option) & ONIG_OPTION_SINGLELINE)
+#define IS_MULTILINE(option)      ((option) & ONIG_OPTION_MULTILINE)
+#define IS_IGNORECASE(option)     ((option) & ONIG_OPTION_IGNORECASE)
+#define IS_EXTEND(option)         ((option) & ONIG_OPTION_EXTEND)
+#define IS_FIND_LONGEST(option)   ((option) & ONIG_OPTION_FIND_LONGEST)
+#define IS_FIND_NOT_EMPTY(option) ((option) & ONIG_OPTION_FIND_NOT_EMPTY)
 #define IS_POSIXLINE(option)      (IS_SINGLELINE(option) && IS_MULTILINE(option))
 #define IS_FIND_CONDITION(option) ((option) & \
-          (REG_OPTION_FIND_LONGEST | REG_OPTION_FIND_NOT_EMPTY))
-#define IS_NOTBOL(option)         ((option) & REG_OPTION_NOTBOL)
-#define IS_NOTEOL(option)         ((option) & REG_OPTION_NOTEOL)
-#define IS_POSIX_REGION(option)   ((option) & REG_OPTION_POSIX_REGION)
+          (ONIG_OPTION_FIND_LONGEST | ONIG_OPTION_FIND_NOT_EMPTY))
+#define IS_NOTBOL(option)         ((option) & ONIG_OPTION_NOTBOL)
+#define IS_NOTEOL(option)         ((option) & ONIG_OPTION_NOTEOL)
+#define IS_POSIX_REGION(option)   ((option) & ONIG_OPTION_POSIX_REGION)
 
 /* OP_SET_OPTION is required for these options.
 #define IS_DYNAMIC_OPTION(option) \
-  (((option) & (REG_OPTION_MULTILINE | REG_OPTION_IGNORECASE)) != 0)
+  (((option) & (ONIG_OPTION_MULTILINE | ONIG_OPTION_IGNORECASE)) != 0)
 */
 /* ignore-case and multibyte status are included in compiled code. */
 #define IS_DYNAMIC_OPTION(option)  0
 
 
-#ifdef NEWLINE
-#undef NEWLINE
-#endif
-#define NEWLINE     '\n'
-#define IS_NULL(p)                    (((void*)(p)) == (void*)0)
-#define IS_NOT_NULL(p)                (((void*)(p)) != (void*)0)
-#define IS_NEWLINE(c)                 ((c) == NEWLINE)
-#define CHECK_NULL_RETURN(p)          if (IS_NULL(p)) return NULL
-#define CHECK_NULL_RETURN_VAL(p,val)  if (IS_NULL(p)) return (val)
-
-#define NULL_UCHARP       ((UChar* )0)
-
 /* bitset */
 #define BITS_PER_BYTE      8
 #define SINGLE_BYTE_SIZE   (1 << BITS_PER_BYTE)
 #define BITS_IN_ROOM       (sizeof(Bits) * BITS_PER_BYTE)
 #define BITSET_SIZE        (SINGLE_BYTE_SIZE / BITS_IN_ROOM)
 
-#ifdef UNALIGNED_WORD_ACCESS
+#ifdef PLATFORM_UNALIGNED_WORD_ACCESS
 typedef unsigned int   Bits;
 #else
 typedef unsigned char  Bits;
@@ -339,42 +329,42 @@ typedef struct _BBuf {
   unsigned int alloc;
 } BBuf;
 
-#define BBUF_INIT(buf,size)    regex_bbuf_init((BBuf* )(buf), (size))
+#define BBUF_INIT(buf,size)    onig_bbuf_init((BBuf* )(buf), (size))
 
 #define BBUF_SIZE_INC(buf,inc) do{\
   (buf)->alloc += (inc);\
   (buf)->p = (UChar* )xrealloc((buf)->p, (buf)->alloc);\
-  if (IS_NULL((buf)->p)) return(REGERR_MEMORY);\
+  if (IS_NULL((buf)->p)) return(ONIGERR_MEMORY);\
 } while (0)
 
 #define BBUF_EXPAND(buf,low) do{\
-  do { (buf)->alloc *= 2; } while ((buf)->alloc < low);\
+  do { (buf)->alloc *= 2; } while ((buf)->alloc < (unsigned int )low);\
   (buf)->p = (UChar* )xrealloc((buf)->p, (buf)->alloc);\
-  if (IS_NULL((buf)->p)) return(REGERR_MEMORY);\
+  if (IS_NULL((buf)->p)) return(ONIGERR_MEMORY);\
 } while (0)
 
 #define BBUF_ENSURE_SIZE(buf,size) do{\
-  int new_alloc = (buf)->alloc;\
-  while (new_alloc < (size)) { new_alloc *= 2; }\
+  unsigned int new_alloc = (buf)->alloc;\
+  while (new_alloc < (unsigned int )(size)) { new_alloc *= 2; }\
   if ((buf)->alloc != new_alloc) {\
     (buf)->p = (UChar* )xrealloc((buf)->p, new_alloc);\
-    if (IS_NULL((buf)->p)) return(REGERR_MEMORY);\
+    if (IS_NULL((buf)->p)) return(ONIGERR_MEMORY);\
     (buf)->alloc = new_alloc;\
   }\
 } while (0)
 
 #define BBUF_WRITE(buf,pos,bytes,n) do{\
   int used = (pos) + (n);\
-  if ((buf)->alloc < used) BBUF_EXPAND((buf),used);\
+  if ((buf)->alloc < (unsigned int )used) BBUF_EXPAND((buf),used);\
   xmemcpy((buf)->p + (pos), (bytes), (n));\
-  if ((buf)->used < used) (buf)->used = used;\
+  if ((buf)->used < (unsigned int )used) (buf)->used = used;\
 } while (0)
 
 #define BBUF_WRITE1(buf,pos,byte) do{\
   int used = (pos) + 1;\
-  if ((buf)->alloc < used) BBUF_EXPAND((buf),used);\
+  if ((buf)->alloc < (unsigned int )used) BBUF_EXPAND((buf),used);\
   (buf)->p[(pos)] = (byte);\
-  if ((buf)->used < used) (buf)->used = used;\
+  if ((buf)->used < (unsigned int )used) (buf)->used = used;\
 } while (0)
 
 #define BBUF_ADD(buf,bytes,n)       BBUF_WRITE((buf),(buf)->used,(bytes),(n))
@@ -384,9 +374,9 @@ typedef struct _BBuf {
 
 /* from < to */
 #define BBUF_MOVE_RIGHT(buf,from,to,n) do {\
-  if ((to) + (n) > (buf)->alloc) BBUF_EXPAND((buf),(to) + (n));\
+  if ((unsigned int )((to)+(n)) > (buf)->alloc) BBUF_EXPAND((buf),(to) + (n));\
   xmemmove((buf)->p + (to), (buf)->p + (from), (n));\
-  if ((to) + (n) > (buf)->used) (buf)->used = (to) + (n);\
+  if ((unsigned int )((to)+(n)) > (buf)->used) (buf)->used = (to) + (n);\
 } while (0)
 
 /* from > to */
@@ -513,6 +503,8 @@ enum OpCode {
   OP_REPEAT_INC_NG,        /* non greedy */
   OP_NULL_CHECK_START,     /* null loop checker start */
   OP_NULL_CHECK_END,       /* null loop checker end   */
+  OP_NULL_CHECK_END_MEMST, /* null loop checker end (with capture status) */
+  OP_NULL_CHECK_END_MEMST_PUSH, /* with capture status and push check-end */
 
   OP_PUSH_POS,             /* (?=...)  start */
   OP_POP_POS,              /* (?=...)  end   */
@@ -549,9 +541,10 @@ typedef int         RepeatNumType;
 #define SIZE_LENGTH        sizeof(LengthType)
 #define SIZE_MEMNUM        sizeof(MemNumType)
 #define SIZE_REPEATNUM     sizeof(RepeatNumType)
-#define SIZE_OPTION        sizeof(RegOptionType)
+#define SIZE_OPTION        sizeof(OnigOptionType)
+#define SIZE_CODE_POINT    sizeof(OnigCodePoint)
 
-#ifdef UNALIGNED_WORD_ACCESS
+#ifdef PLATFORM_UNALIGNED_WORD_ACCESS
 #define GET_RELADDR_INC(addr,p) do{\
   addr = *((RelAddrType* )(p));\
   (p) += SIZE_RELADDR;\
@@ -578,7 +571,7 @@ typedef int         RepeatNumType;
 } while(0)
 
 #define GET_OPTION_INC(option,p) do{\
-  option = *((RegOptionType* )(p));\
+  option = *((OnigOptionType* )(p));\
   (p) += SIZE_OPTION;\
 } while(0)
 #else
@@ -599,8 +592,10 @@ typedef int         RepeatNumType;
 
 #define SERIALIZE_BUFSIZE            SIZEOF_INT
 
-#endif  /* UNALIGNED_WORD_ACCESS */
+#endif  /* PLATFORM_UNALIGNED_WORD_ACCESS */
 
+/* code point's address must be aligned address. */
+#define GET_CODE_POINT(code,p)   code = *((OnigCodePoint* )(p))
 #define GET_BYTE_INC(byte,p) do{\
   byte = *(p);\
   (p)++;\
@@ -642,49 +637,49 @@ typedef int         RepeatNumType;
 
 
 typedef struct {
-  CodePoint esc;
-  CodePoint anychar;
-  CodePoint anytime;
-  CodePoint zero_or_one_time;
-  CodePoint one_or_more_time;
-  CodePoint anychar_anytime;
-} RegMetaCharTableType;
+  UChar esc;
+  UChar anychar;
+  UChar anytime;
+  UChar zero_or_one_time;
+  UChar one_or_more_time;
+  UChar anychar_anytime;
+} OnigMetaCharTableType;
 
-extern RegMetaCharTableType RegMetaCharTable;
+extern OnigMetaCharTableType OnigMetaCharTable;
 
-#define MC_ESC               RegMetaCharTable.esc
-#define MC_ANYCHAR           RegMetaCharTable.anychar
-#define MC_ANYTIME           RegMetaCharTable.anytime
-#define MC_ZERO_OR_ONE_TIME  RegMetaCharTable.zero_or_one_time
-#define MC_ONE_OR_MORE_TIME  RegMetaCharTable.one_or_more_time
-#define MC_ANYCHAR_ANYTIME   RegMetaCharTable.anychar_anytime
+#define MC_ESC               OnigMetaCharTable.esc
+#define MC_ANYCHAR           OnigMetaCharTable.anychar
+#define MC_ANYTIME           OnigMetaCharTable.anytime
+#define MC_ZERO_OR_ONE_TIME  OnigMetaCharTable.zero_or_one_time
+#define MC_ONE_OR_MORE_TIME  OnigMetaCharTable.one_or_more_time
+#define MC_ANYCHAR_ANYTIME   OnigMetaCharTable.anychar_anytime
 
 
-#ifdef REG_DEBUG
+#ifdef ONIG_DEBUG
 
 typedef struct {
   short int opcode;
   char*     name;
   short int arg_type;
-} RegOpInfoType;
+} OnigOpInfoType;
 
-extern RegOpInfoType RegOpInfo[];
+extern OnigOpInfoType OnigOpInfo[];
 
-extern void regex_print_compiled_byte_code P_((FILE* f, UChar* bp, UChar** nextp));
+extern void onig_print_compiled_byte_code P_((FILE* f, UChar* bp, UChar** nextp));
 
-#ifdef REG_DEBUG_STATISTICS
-extern void regex_statistics_init P_((void));
-extern void regex_print_statistics P_((FILE* f));
+#ifdef ONIG_DEBUG_STATISTICS
+extern void onig_statistics_init P_((void));
+extern void onig_print_statistics P_((FILE* f));
 #endif
 #endif
 
-extern char* regex_error_code_to_format P_((int code));
-extern void  regex_snprintf_with_pattern PV_((char buf[], int bufsize, RegCharEncoding enc, char* pat, char* pat_end, char *fmt, ...));
-extern UChar* regex_strdup P_((UChar* s, UChar* end));
-extern int  regex_bbuf_init P_((BBuf* buf, int size));
-extern int  regex_alloc_init P_((regex_t** reg, RegOptionType option, RegCharEncoding enc, RegSyntaxType* syntax));
-extern int  regex_compile P_((regex_t* reg, UChar* pattern, UChar* pattern_end, RegErrorInfo* einfo));
-extern void regex_chain_reduce P_((regex_t* reg));
-extern int  regex_is_in_code_range P_((UChar* p, CodePoint code));
+extern char* onig_error_code_to_format P_((int code));
+extern void  onig_snprintf_with_pattern PV_((char buf[], int bufsize, OnigEncoding enc, char* pat, char* pat_end, char *fmt, ...));
+extern UChar* onig_strdup P_((UChar* s, UChar* end));
+extern int  onig_bbuf_init P_((BBuf* buf, int size));
+extern int  onig_alloc_init P_((regex_t** reg, OnigOptionType option, OnigEncoding enc, OnigSyntaxType* syntax));
+extern int  onig_compile P_((regex_t* reg, UChar* pattern, UChar* pattern_end, OnigErrorInfo* einfo));
+extern void onig_chain_reduce P_((regex_t* reg));
+extern int  onig_is_in_code_range P_((UChar* p, OnigCodePoint code));
 
 #endif /* REGINT_H */
